@@ -1,8 +1,9 @@
 /*
  * Project    : ipv6calc
  * File       : ipv6logconvhelp.c
- * Version    : $Id: ipv6logconvhelp.c,v 1.4 2003/11/21 09:36:58 peter Exp $
- * Copyright  : 2002 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: ipv6logconvhelp.c,v 1.7 2005/02/13 11:50:30 peter Exp $
+ * Copyright  : 2002-2005 by Peter Bieringer <pb (at) bieringer.de>
+ * License    : GNU GPL v2
  *
  * Information:
  *  Help library
@@ -41,9 +42,16 @@ void ipv6logconv_printhelp(void) {
 	fprintf(stderr, " General:\n");
 	fprintf(stderr, "  [-d|--debug <debug value>] : debug value (bitwise like)\n");
 	fprintf(stderr, "  [-q|--quiet] : be more quiet\n");
+	fprintf(stderr, "  [-n|--nocache] : disable caching\n");
+	fprintf(stderr, "  [-c|--cachelimit <value>] : set cache limit\n");
+	fprintf(stderr, "                               default: %d\n", cache_lru_limit);
+	fprintf(stderr, "                               maximum: %d\n", CACHE_LRU_SIZE);
 	fprintf(stderr, " Output:\n");
 	fprintf(stderr, "  [--out <output type>] : specify output type\n");
-	fprintf(stderr, "  Available output types:  --out -?\n");
+	fprintf(stderr, "   addrtype       : Address type\n");
+	fprintf(stderr, "   ouitype        : OUI (IEEE) type\n");
+	fprintf(stderr, "   ipv6addrtype   : IPv6 address type\n");
+	fprintf(stderr, "   any            : any type\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, " Takes data from stdin, proceed it to stdout\n");
 	fprintf(stderr, "\n");
